package com.cnbm.qualityPlanning.common;

import com.cnbm.qualityPlanning.entity.ControlLimit;
import com.cnbm.qualityPlanning.entity.Point;
import io.swagger.models.auth.In;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

/**
 * @Desc: "检测参数统计受控  检验类"
 * @Author: caixiang
 * @DATE: 2022/7/11 15:27
 */
public class StatisticalControlledTest {
    private static final Integer rule1Number = 1;
    private static final Integer rule2Number = 2;
    private static final Integer rule3Number = 3;
    private static final Integer rule4Number = 4;
    private static final Integer rule5Number = 5;
    private static final Integer rule6Number = 6;
    private static final Integer rule7Number = 7;
    private static final Integer rule8Number = 8;
    private static final Integer rule9Number = 9;
    private static final Integer rule10Number = 10;

    public static List<Point> createData(){
        Point point = new Point(1,new Double(1));
        Point point2 = new Point(2,new Double(2));
        Point point3 = new Point(3,new Double(3));
        Point point4 = new Point(4,new Double(4));
        Point point5 = new Point(5,new Double(5));
        Point point6 = new Point(6,new Double(6));
        Point point7 = new Point(7,new Double(7));
        Point point8 = new Point(8,new Double(8));
        Point point9 = new Point(9,new Double(9));
        Point point10 = new Point(10,new Double(10));
        Point point11 = new Point(11,new Double(11));
        Point point12 = new Point(12,new Double(12));
        List<Point> list = new ArrayList<>();
        list.add(point);
        list.add(point2);
        list.add(point3);
        list.add(point4);
        list.add(point5);
        list.add(point6);
        list.add(point7);
        list.add(point8);
        list.add(point9);
        list.add(point10);
        list.add(point11);
        list.add(point12);
        return list;
    }

    public static void main(String[] args) {
        List<Point> list = createData();
        ControlLimit controlLimit = new ControlLimit(new Double(12),new Double(2),new Double(1));

        //TEST FOR RULE1
        Boolean aBoolean1 = rule1(list, controlLimit);

        //TEST FOR RULE2
        Boolean aBoolean2 = rule2(list, controlLimit, 9);
        System.out.println();


        //TEST FOR RULE3
        Boolean aBoolean3 = rule3(list, 3);
        System.out.println();
    }

    /**
     * name : 规则1
     * desc : 控制图上有 1 个点位于三倍标准差以外(对中心线来说)（母体的 $\sigma$ ）
     *
     * return : 返回的 就是不满足 规则1  的点。
     * */
    private static Boolean rule1(List<Point> data, ControlLimit controlLimit){
        for(Point i:data){
            if(i.getValue() > controlLimit.getUCL() || i.getValue() < controlLimit.getLCL()){
                i.getUnsatisfiedRules().add(1);
            }
        }
        return true;
    }

    private static void markUnsatisfiedRulesByKeys(List<Point> data,List<Integer> keys,Integer ruleNumber){
        for (Integer key:keys){
            data.get(key).getUnsatisfiedRules().add(ruleNumber);
        }
    }

    /**
     * name : 规则2
     * desc : 连续 n 点落在中心线的用一侧 (默认：9)
     *
     * return : 返回的 就是不满足 规则2  的点。
     * */
    private static Boolean rule2(List<Point> data, ControlLimit controlLimit,Integer n){
        List<Point> upList = new ArrayList<>();
        List<Point> downList = new ArrayList<>();
        for(Point i:data){
            if(i.getValue() > controlLimit.getCL()){
                upList.add(i);
            }else if(i.getValue() < controlLimit.getCL()){
                downList.add(i);
            }
        }

        //上侧的情况
        Integer upi = 0;
        Integer uptime = 0;
        //标注不合格的 参数
        List<Point> forMarkPoints = new ArrayList<>();
        while((upi+1) < upList.size()){
            //if((upList.get(upi).getPosition()+1)==upList.get(upi+1).getPosition()){
            if(isBetween(upList.get(upi),upList.get(upi+1))){
                uptime +=1;
                forMarkPoints.add(upList.get(upi));
                //只要到达n 了，就return，防止中间段到达 后面又被重置了
                if(uptime>=n){
                    //1.
                    for (Point key:forMarkPoints){
                        for(Point j:data){
                            if(key.getPosition()== j.getPosition()){
                                j.getUnsatisfiedRules().add(rule2Number);
                                continue;
                            }
                        }
                    }
                    //2.
//                    markUnsatisfiedRulesByKeys(data,forMarkKey,rule2Number);

                    return true;
                }
            }else {
                forMarkPoints = new ArrayList<>();
                uptime = 0;
            }
            upi++;
        }


        //下侧的情况
        Integer downi = 0;
        Integer downtime = 0;
        forMarkPoints = new ArrayList<>();
        while((downi+1) < downList.size()){
            if(isBetween(downList.get(upi),downList.get(upi+1))){
                downtime +=1;
                forMarkPoints.add(downList.get(downi));
                if(downtime>=n){

                    for (Point key:forMarkPoints){
                        for(Point j:data){
                            if(key.getPosition()== j.getPosition()){
                                j.getUnsatisfiedRules().add(rule2Number);
                                continue;
                            }
                        }
                    }

                    return true;
                }
            }else {
                forMarkPoints = new ArrayList<>();
                downtime = 0;
            }
            downi++;
        }


        return false;
    }

    /**
     * name : 规则3
     * desc : 连续 n 点递增或者递减 (默认：6)
     *
     * return : 返回的 就是不满足 规则2  的点。
     * */
    private static Boolean rule3(List<Point> data, Integer n){

        //递增情况
        Integer upi = 0;
        Integer uptime = 0;
        List<Integer> forMarkKey = new ArrayList<>();
        while((upi+1) < data.size()){
            if((isBetween( data.get(upi),data.get(upi+1) )) && isSmall( data.get(upi),data.get(upi+1) ) ){
                uptime +=1;
                forMarkKey.add(upi);
                if(uptime>=n){
                    for(Integer key:forMarkKey){
                        data.get(key).getUnsatisfiedRules().add(rule3Number);
                    }

                    return true;
                }
            }else {
                uptime = 0;
                forMarkKey = new ArrayList<>();
            }
            upi++;
        }


        //递减情况
        Integer downi = 0;
        Integer downtime = 0;
        forMarkKey = new ArrayList<>();
        while((downi+1) < data.size()){
            if((isBetween( data.get(downi),data.get(downi+1) )) && isBig( data.get(downi),data.get(downi+1) ) ){
                downtime +=1;
                forMarkKey.add(downi);
                if(downtime>=n){
                    for(Integer key:forMarkKey){
                        data.get(key).getUnsatisfiedRules().add(rule3Number);
                    }

                    return true;
                }
            }else {
                downtime = 0;
                forMarkKey = new ArrayList<>();
            }
            downi++;
        }

        return false;
    }




    /**
     * 如果 这两个点是相邻的  ==》 true
     * */
    private static boolean isBetween(Point head,Point after){
        return (head.getPosition()+1) == after.getPosition();
    }
    /**
     * 如果 head 小于 after ==》  true
     * */
    private static boolean isSmall(Point head,Point after){
        return (head.getValue()- after.getValue()) < 0;
    }
    /**
     * 如果 head 大于 after ==》  true
     * */
    private static boolean isBig(Point head,Point after){
        return (head.getValue()- after.getValue()) > 0;
    }

}
